package com.sun.corba.se.impl.util;

import java.util.Collections;
import java.util.WeakHashMap;
import java.util.Map;
import java.security.AccessController;
import java.security.PrivilegedAction;
import sun.corba.Bridge;

class JDKClassLoader
{
    private static final JDKClassLoader.JDKClassLoaderCache classCache;
    private static final Bridge bridge;
    
    JDKClassLoader() {
    }
    
    static Class loadClass(Class clazz, String s) throws ClassNotFoundException {
        if (s == null) {
            throw new NullPointerException();
        }
        if (s.length() == 0) {
            throw new ClassNotFoundException();
        }
        ClassLoader loader;
        if (clazz != null) {
            loader = clazz.getClassLoader();
        }
        else {
            loader = JDKClassLoader.bridge.getLatestUserDefinedLoader();
        }
        Object key = JDKClassLoader.classCache.createKey(s, loader);
        if (JDKClassLoader.classCache.knownToFail(key)) {
            throw new ClassNotFoundException(s);
        }
        try {
            return Class.forName(s, false, loader);
        }
        catch (ClassNotFoundException ex) {
            JDKClassLoader.classCache.recordFailure(key);
            throw ex;
        }
    }
    
    static {
        classCache = new JDKClassLoader.JDKClassLoaderCache();
        bridge = (Bridge)AccessController.doPrivileged(new PrivilegedAction() {
            @Override
            public Object run() {
                return Bridge.get();
            }
        });
    }
    
    private static class JDKClassLoaderCache
    {
        private final Map cache;
        private static final Object KNOWN_TO_FAIL;
        
        private JDKClassLoaderCache() {
            this.cache = Collections.synchronizedMap(new WeakHashMap());
        }
        
        public final void recordFailure(Object o) {
            this.cache.put(o, JDKClassLoader.JDKClassLoaderCache.KNOWN_TO_FAIL);
        }
        
        public final Object createKey(String s, ClassLoader classLoader) {
            return new JDKClassLoader.JDKClassLoaderCache.CacheKey(s, classLoader);
        }
        
        public final boolean knownToFail(Object o) {
            return this.cache.get(o) == JDKClassLoader.JDKClassLoaderCache.KNOWN_TO_FAIL;
        }
        
        static {
            KNOWN_TO_FAIL = new Object();
        }
        
        private static class CacheKey
        {
            String className;
            ClassLoader loader;
            
            public CacheKey(String className, ClassLoader loader) {
                this.className = className;
                this.loader = loader;
            }
            
            @Override
            public int hashCode() {
                if (this.loader == null) {
                    return this.className.hashCode();
                }
                return this.className.hashCode() ^ this.loader.hashCode();
            }
            
            @Override
            public boolean equals(Object o) {
                try {
                    if (o == null) {
                        return false;
                    }
                    JDKClassLoader.JDKClassLoaderCache.CacheKey cacheKey = (JDKClassLoader.JDKClassLoaderCache.CacheKey)o;
                    return this.className.equals(cacheKey.className) && this.loader == cacheKey.loader;
                }
                catch (ClassCastException ex) {
                    return false;
                }
            }
        }
    }
}
